@@ -35,20 +35,12 @@ _SERVICE_NAME = "org.laptop.Activity"
 _SERVICE_PATH = "/org/laptop/Activity"
 _SERVICE_INTERFACE = "org.laptop.Activity"
 
-def get_sugar_window_type(wnck_window):
-    window = gtk.gdk.window_foreign_new(wnck_window.get_xid())
-    prop_info = window.property_get('_SUGAR_WINDOW_TYPE', 'STRING')
-    if prop_info is None:
-        return None
-    else:
-        return prop_info[2]
-
 class Activity(gobject.GObject):
     """Activity which appears in the "Home View" of the Sugar shell
-    
+
     This class stores the Sugar Shell's metadata regarding a
     given activity/application in the system.  It interacts with
-    the sugar.activity.* modules extensively in order to 
+    the sugar.activity.* modules extensively in order to
     accomplish its tasks.
     """
 
@@ -61,14 +53,14 @@ class Activity(gobject.GObject):
 
     def __init__(self, activity_info, activity_id, window=None):
         """Initialise the HomeActivity
-        
+
         activity_info -- sugar.activity.registry.ActivityInfo instance,
             provides the information required to actually
             create the new instance.  This is, in effect,
             the "type" of activity being created.
         activity_id -- unique identifier for this instance
             of the activity type
-        window -- Main WnckWindow of the activity 
+        window -- Main WnckWindow of the activity
         """
         gobject.GObject.__init__(self)
 
@@ -104,7 +96,7 @@ class Activity(gobject.GObject):
 
     def get_service(self):
         """Get the activity service
-        
+
         Note that non-native Sugar applications will not have
         such a service, so the return value will be None in
         those cases.
@@ -123,7 +115,7 @@ class Activity(gobject.GObject):
         """Retrieve the activity's icon (file) name"""
         if self.is_journal():
             icon_theme = gtk.icon_theme_get_default()
-            info = icon_theme.lookup_icon('activity-journal', 
+            info = icon_theme.lookup_icon('activity-journal',
                                           gtk.ICON_SIZE_SMALL_TOOLBAR, 0)
             if not info:
                 return None
@@ -134,11 +126,11 @@ class Activity(gobject.GObject):
             return self._activity_info.get_icon()
         else:
             return None
-    
+
     def get_icon_color(self):
         """Retrieve the appropriate icon colour for this activity
-        
-        Uses activity_id to index into the PresenceService's 
+
+        Uses activity_id to index into the PresenceService's
         set of activity colours, if the PresenceService does not
         have an entry (implying that this is not a Sugar-shared application)
         uses the local user's profile colour for the icon.
@@ -158,10 +150,10 @@ class Activity(gobject.GObject):
         else:
             client = gconf.client_get_default()
             return XoColor(client.get_string("/desktop/sugar/user/color"))
-        
+
     def get_activity_id(self):
         """Retrieve the "activity_id" passed in to our constructor
-        
+
         This is a "globally likely unique" identifier generated by
         sugar.util.unique_id
         """
@@ -176,12 +168,12 @@ class Activity(gobject.GObject):
 
     def get_window(self):
         """Retrieve the X-windows root window of this application
-        
-        This was stored by the set_window method, which was 
-        called by HomeModel._add_activity, which was called 
+
+        This was stored by the set_window method, which was
+        called by HomeModel._add_activity, which was called
         via a callback that looks for all 'window-opened'
         events.
-        
+
         HomeModel currently uses a dbus service query on the
         activity to determine to which HomeActivity the newly
         launched window belongs.
@@ -201,8 +193,8 @@ class Activity(gobject.GObject):
 
     def get_launch_time(self):
         """Return the time at which the activity was first launched
-        
-        Format is floating-point time.time() value 
+
+        Format is floating-point time.time() value
         (seconds since the epoch)
         """
         return self._launch_time
@@ -218,6 +210,13 @@ class Activity(gobject.GObject):
         else:
             return self._activity_info.get_path()
 
+    def get_activity_name(self):
+        """Returns the activity's bundle name"""
+        if self._activity_info is None:
+            return None
+        else:
+            return self._activity_info.get_name()
+
     def equals(self, activity):
         if self._activity_id and activity.get_activity_id():
             return self._activity_id == activity.get_activity_id()
@@ -267,20 +266,20 @@ class Activity(gobject.GObject):
 
     def _set_active_success(self):
         pass
-    
+
     def _set_active_error(self, err):
-        logging.error("set_active() failed: %s" % err)
+        logging.error("set_active() failed: %s", err)
 
 class ShellModel(gobject.GObject):
     """Model of the shell (activity management)
-    
+
     The ShellModel is basically the point of registration
     for all running activities within Sugar.  It traps
     events that tell the system there is a new activity
     being created (generated by the activity factories),
     or removed, as well as those which tell us that the
     currently focussed activity has changed.
-    
+
     The HomeModel tracks a set of HomeActivity instances,
     which are tracking the window to activity mappings
     the activity factories have set up.
@@ -288,7 +287,7 @@ class ShellModel(gobject.GObject):
 
     __gsignals__ = {
         'activity-added':          (gobject.SIGNAL_RUN_FIRST,
-                                    gobject.TYPE_NONE, 
+                                    gobject.TYPE_NONE,
                                    ([gobject.TYPE_PYOBJECT])),
         'activity-removed':        (gobject.SIGNAL_RUN_FIRST,
                                     gobject.TYPE_NONE,
@@ -300,13 +299,13 @@ class ShellModel(gobject.GObject):
                                     gobject.TYPE_NONE,
                                    ([gobject.TYPE_PYOBJECT])),
         'launch-started':          (gobject.SIGNAL_RUN_FIRST,
-                                    gobject.TYPE_NONE, 
+                                    gobject.TYPE_NONE,
                                    ([gobject.TYPE_PYOBJECT])),
         'launch-completed':        (gobject.SIGNAL_RUN_FIRST,
-                                    gobject.TYPE_NONE, 
+                                    gobject.TYPE_NONE,
                                    ([gobject.TYPE_PYOBJECT])),
         'launch-failed':           (gobject.SIGNAL_RUN_FIRST,
-                                    gobject.TYPE_NONE, 
+                                    gobject.TYPE_NONE,
                                    ([gobject.TYPE_PYOBJECT]))
     }
 
@@ -355,6 +354,12 @@ class ShellModel(gobject.GObject):
         if old_level == new_level:
             return
 
+        if old_level != self.ZOOM_ACTIVITY:
+            screen = gtk.gdk.screen_get_default()
+            active_window_type = screen.get_active_window().get_type_hint()
+            if active_window_type != gtk.gdk.WINDOW_TYPE_HINT_DESKTOP:
+                return
+
         self._zoom_level = new_level
         if new_level is not self.ZOOM_ACTIVITY:
             self._desktop_level = new_level
@@ -429,15 +434,15 @@ class ShellModel(gobject.GObject):
         self._active_activity = home_activity
         self.emit('active-activity-changed', self._active_activity)
 
-    def __iter__(self): 
+    def __iter__(self):
         return iter(self._activities)
-        
+
     def __len__(self):
         return len(self._activities)
-        
+
     def __getitem__(self, i):
         return self._activities[i]
-        
+
     def index(self, obj):
         return self._activities.index(obj)
 
@@ -457,19 +462,26 @@ class ShellModel(gobject.GObject):
             if activity_id:
                 home_activity = self.get_activity_by_id(activity_id)
 
+                xid = window.get_xid()
+                gdk_window = gtk.gdk.window_foreign_new(xid)
+                gdk_window.set_decorations(0)
+
+                window.maximize()
+
             if not home_activity:
                 home_activity = Activity(activity_info, activity_id, window)
                 self._add_activity(home_activity)
             else:
                 home_activity.set_window(window)
 
-            if get_sugar_window_type(window) != 'launcher':
+            if wm.get_sugar_window_type(window) != 'launcher':
                 home_activity.props.launching = False
-                self.emit('launch-completed', home_activity)
+                if not home_activity.is_journal():
+                    self.emit('launch-completed', home_activity)
 
                 startup_time = time.time() - home_activity.get_launch_time()
-                logging.debug('%s launched in %f seconds.' %
-                                   (home_activity.get_type(), startup_time))
+                logging.debug('%s launched in %f seconds.',
+                    home_activity.get_type(), startup_time)
 
             if self._active_activity is None:
                 self._set_active_activity(home_activity)
@@ -531,7 +543,7 @@ class ShellModel(gobject.GObject):
         if home_activity:
             self._remove_activity(home_activity)
         else:
-            logging.error('Model for window %d does not exist.' % xid)
+            logging.error('Model for window %d does not exist.', xid)
 
     def notify_launch(self, activity_id, service_name):
         registry = get_registry()
@@ -556,13 +568,13 @@ class ShellModel(gobject.GObject):
     def notify_launch_failed(self, activity_id):
         home_activity = self.get_activity_by_id(activity_id)
         if home_activity:
-            logging.debug("Activity %s (%s) launch failed" % \
-                          (activity_id, home_activity.get_type()))
+            logging.debug("Activity %s (%s) launch failed", activity_id,
+                home_activity.get_type())
             home_activity.props.launching = False
             self._remove_activity(home_activity)
         else:
-            logging.error('Model for activity id %s does not exist.'
-                          % activity_id)
+            logging.error('Model for activity id %s does not exist.',
+                activity_id)
 
         self.emit('launch-failed', home_activity)
 
@@ -570,15 +582,16 @@ class ShellModel(gobject.GObject):
         home_activity = self.get_activity_by_id(activity_id)
 
         if not home_activity:
-            logging.debug('Activity %s has been closed already.' % activity_id)
+            logging.debug('Activity %s has been closed already.', activity_id)
             return False
 
         if home_activity.props.launching:
-            logging.debug('Activity %s still launching, assuming it failed...'
-                          % activity_id)
+            logging.debug('Activity %s still launching, assuming it failed.',
+                activity_id)
             self.notify_launch_failed(activity_id)
         return False
 
+
 _model = None
 
 def get_model():
